import logging
from typing import Any, Dict, Optional, Type, Tuple

from iqbench.evaluation.evaluation_basic import EvaluationBasic
from iqbench.evaluation.evaluation_judge import EvaluationWithJudge
from iqbench.evaluation.evaluation_base import EvaluationBase
from iqbench.models.llm_judge import LLMJudge


class EvaluationFactory:
    def __init__(self):
        self.logger = logging.getLogger(self.__class__.__name__)

        self.logger.info(f"EnsembleFactory initialized.")

    def which_evaluator(
        self,
        dataset_name: str,
        strategy_name: Optional[str] = None,
        ensemble: bool = False,
    ) -> Optional[Type[EvaluationBase]]:

        if "bp" in dataset_name.lower():
            if not ensemble and strategy_name is None:
                raise ValueError(
                    "strategy_name must be provided for non-ensemble evaluations."
                )
            if strategy_name is not None and strategy_name.lower() == "classification":
                return EvaluationBasic
            else:
                return EvaluationWithJudge

        return EvaluationBasic

    def create_evaluator(
        self,
        dataset_name: str,
        ensemble: bool = False,
        strategy_name: Optional[str] = None,
        judge_model_object: Optional[LLMJudge] = None,
        judge_model_name: Optional[str] = "mistralai/Mistral-7B-Instruct-v0.3",
        judge_param_set_number: Optional[int] = None,
        prompt_number: Optional[int] = 1,
        prompt: Optional[str] = None,
    ) -> EvaluationBase:

        if not ensemble and strategy_name is None:
            raise ValueError(
                "strategy_name must be provided for non-ensemble evaluations."
            )

        evaluator_cls = self.which_evaluator(
            dataset_name=dataset_name, strategy_name=strategy_name, ensemble=ensemble
        )

        evaluator = evaluator_cls(
            judge_model_object=judge_model_object, judge_model_name=judge_model_name
        )

        if isinstance(evaluator, EvaluationWithJudge) and prompt_number is not None:
            evaluator.prompt_number = prompt_number
        if isinstance(evaluator, EvaluationWithJudge) and prompt is not None:
            evaluator.prompt = prompt
        if (
            isinstance(evaluator, EvaluationWithJudge)
            and judge_param_set_number is not None
        ):
            evaluator.judge_param_set_number = judge_param_set_number

        return evaluator



from traitlets import List
import os
from typing import Optional, Dict, Any


from iqbench.strategies.strategy_base import StrategyBase
from iqbench.technical.content import ImageContent, TextContent
from iqbench.technical.utils import get_field
from iqbench.technical.response_schema import DescriptionResponseSchema, ResponseSchema
from iqbench.models.vllm import VLLM
from iqbench.technical.configs.dataset_config import DatasetConfig


class ContrastiveStrategy(StrategyBase):
    def __init__(
        self,
        dataset_name: str,
        model: VLLM,
        dataset_config: DatasetConfig,
        results_dir: str,
        strategy_name: str,
        prompt_number: int,
        param_set_number: Optional[int] = None,
    ):
        super().__init__(
            dataset_name,
            model,
            dataset_config,
            results_dir,
            strategy_name,
            prompt_number=prompt_number,
            param_set_number=param_set_number,
        )

        self.descriptions_prompt = self.get_prompt(f"describe", self.prompt_number)
        self.contrast_example_prompt = self.get_prompt(
            f"contrast_example", self.prompt_number
        )
        self.descriptions_path = os.path.join(self.results_dir, "descriptions.json")

    def _execute_problem(
        self, problem_id: str
    ) -> list[Dict[str, str], str, Optional[Dict[str, str]]]:
        """
        Executes the logic for a single contrastive problem.
        """

        problem_descriptions_dict = {}
        contrastive_prompt = (
            f"{self.descriptions_prompt}\n{self.contrast_example_prompt}"
        )

        if self.config.category in {"BP", "choice_only"}:
            collected_descriptions = []

            for i in range(self.config.num_choices):

                if self.config.category == "BP":
                    string_index = f"{i}"
                    if i >= self.config.num_choices // 2:
                        break
                    choice_image_input_1 = self.get_choice_image(
                        problem_id, image_index=i
                    )
                    choice_image_input_2 = self.get_choice_image(
                        problem_id, image_index=i + 6
                    )

                    contents_to_send_descriptions = [
                        TextContent(contrastive_prompt),
                        ImageContent(choice_image_input_1),
                        ImageContent(choice_image_input_2),
                    ]

                elif self.config.category == "choice_only":
                    string_index = chr(65 + i)
                    choice_image_input = self.get_blackout_image(
                        problem_id, image_index=string_index
                    )

                    contents_to_send_descriptions = [
                        TextContent(contrastive_prompt),
                        ImageContent(choice_image_input),
                    ]

                description_response = self.model.ask(
                    contents_to_send_descriptions, schema=DescriptionResponseSchema
                )

                desc_text = get_field(description_response, "description", None)
                if desc_text:
                    collected_descriptions.append(str(desc_text))
                    problem_descriptions_dict[string_index] = str(desc_text)

            all_descriptions_text = "\n\n".join(collected_descriptions)

            prompt = f"{self.main_prompt}\nDescriptions:\n{all_descriptions_text}\n{self.example_prompt}"
            contents_to_send = [TextContent(prompt)]

        elif self.config.category == "standard":
            question_image_input = self.get_question_image(problem_id)

            contents_to_send_descriptions = [
                TextContent(contrastive_prompt),
                ImageContent(question_image_input),
            ]

            description_response = self.model.ask(
                contents_to_send_descriptions, schema=DescriptionResponseSchema
            )

            desc_text = get_field(description_response, "description", None)
            if desc_text:
                problem_descriptions_dict["question_image"] = desc_text
                all_descriptions_text = desc_text
            else:
                all_descriptions_text = ""

            prompt = f"{self.main_prompt}\nDescription of question image:\n{all_descriptions_text}\n{self.example_prompt}"
            choice_panel_input = self.get_choice_panel(problem_id)

            if choice_panel_input is None:
                self.logger.error(
                    f"Could not get choice panel for problem {problem_id}. Skipping image content."
                )
                contents_to_send = [TextContent(prompt)]
            else:
                contents_to_send = [
                    TextContent(prompt),
                    ImageContent(choice_panel_input),
                ]

        response = self.model.ask(contents_to_send, schema=ResponseSchema)

        return response, problem_id, problem_descriptions_dict



from typing import Any
import pandas as pd
import json
import logging
import os
from pathlib import Path

from iqbench.evaluation.evaluation_base import EvaluationBase

logger = logging.getLogger(__name__)


class EvaluationBasic(EvaluationBase):
    def evaluate_single_answer(self, answer: str, key: str) -> float:
        score = "Right" if answer == key else "Wrong"
        return score

    def evaluate(
        self, output_df: pd.DataFrame, key_dict: dict, dataset_category: str = None
    ):

        for index, row in output_df.iterrows():
            answer = row.get("answer")
            id_ = str(row["problem_id"])

            if id_ not in key_dict:
                logger.info(f"ID {id_} not found in key file.")
                output_df.at[index, "score"] = "Problem id not found in key"
                output_df.at[index, "key"] = "Key missing"
                continue

            key = key_dict[id_].strip().upper()

            if answer is None or pd.isna(answer) or answer.strip() == "":
                output_df.at[index, "score"] = "No answer provided"
                output_df.at[index, "key"] = key
                continue

            score = self.evaluate_single_answer(answer, key)
            output_df.at[index, "score"] = score
            output_df.at[index, "key"] = key

    def calculate_metrics(self, evaluated_df):
        total = len(evaluated_df)

        if "score" in evaluated_df.columns:
            correct = (evaluated_df["score"] == "Right").sum()
            bin_counts = evaluated_df.groupby("score").size().to_dict()
        else:
            correct = 0
            bin_counts = {}

        accuracy = correct / total if total > 0 else 0.0

        if "confidence" in evaluated_df.columns and "score" in evaluated_df.columns:
            avg_confidence = (
                evaluated_df.groupby("score")["confidence"].mean().to_dict()
            )
            median_confidence = (
                evaluated_df.groupby("score")["confidence"].median().to_dict()
            )
        else:
            avg_confidence = {}
            median_confidence = {}

        return {
            "total": total,
            "bin_counts": bin_counts,
            "accuracy": accuracy,
            "avg_confidence": avg_confidence,
            "median_confidence": median_confidence,
        }



import os
import json
import streamlit as st
import pandas as pd

from iqbench.visualisation.plots import create_countplot, create_confidence_boxplot
from iqbench.visualisation.data_handling import get_present_colors, load_json_safe
from iqbench.visualisation.ui_helpers import (
    shorten_model_name,
    safe_display,
    render_markdown,
)


def show_csv_preview(df: pd.DataFrame) -> None:
    if df.empty:
        st.info("No data to display.")
        return
    st.subheader("Results CSV Preview")
    cols_to_drop = ["filter_id", "ensemble", "judge_filter_id"]
    if df["ensemble"].all():
        cols_to_drop.append("model_name")
        cols_to_drop.append("strategy_name")
        cols_to_drop.append("confidence")
    else:
        cols_to_drop.append("type_name")
    st.dataframe(df.drop(columns=cols_to_drop, errors="ignore"))


def setup_layout() -> None:
    st.markdown(
        """
        <style>
            .main { max-width: 90% !important; }
            .block-container {
                padding-top: 2rem;
                max-width: 90% !important;
            }
        </style>
        """,
        unsafe_allow_html=True,
    )
    st.title("Experiment Results Overview")


def render_cell(value, font_size=18, bold=False):
    weight = "font-weight:bold;" if bold else ""
    st.markdown(
        f"<p style='{weight} font-size:{font_size}px;'>" f"{safe_display(value)}</p>",
        unsafe_allow_html=True,
    )


def centered_st(func):
    def wrapper(*args, **kwargs):
        _, center, _ = st.columns([1, 8, 1])
        with center:
            return func(*args, **kwargs)

    return wrapper


# plotting


@centered_st
def plot_judged_answers(df, **kwargs):
    st.subheader("Scored Answers Summary")
    fig = create_countplot(df, **kwargs)
    st.pyplot(fig) if fig else st.info("No valid data to display.")


@centered_st
def plot_confidence(df, **kwargs):
    st.subheader("Model Confidence Distribution")
    fig = create_confidence_boxplot(df, **kwargs)
    st.pyplot(fig) if fig else st.info("No valid data to display.")


# problem display


def show_chosen_problem(
    df, problem_id, dataset_name, strategy_name, strategy_col="strategy_name"
):
    if df.empty:
        st.info("No valid data to display.")
        return

    row = df[
        (df["problem_id"] == problem_id)
        & (df["dataset_name"] == dataset_name)
        & (df[strategy_col] == strategy_name)
    ].squeeze()

    col1, col2 = st.columns(2)

    with col1:
        img_path = (
            os.path.join(
                "data", dataset_name, "problems", str(problem_id), "question_panel.png"
            )
            if dataset_name
            else None
        )

        if img_path and os.path.exists(img_path):
            st.image(img_path, caption=f"{dataset_name} – Problem {problem_id}")
        else:
            st.warning("Problem image not available.")

    with col2:
        for k in ["answer", "confidence", "key", "score", "rationale"]:
            render_markdown(k.replace("_", " ").title(), row.get(k))


# evaluation summary


def display_evaluation_summary(
    df, dataset_name, strategy_name, strategy_col="strategy_name", is_ensemble=False
):

    df = df[(df["dataset_name"] == dataset_name) & (df[strategy_col] == strategy_name)]

    if df.empty:
        st.info("No data for this dataset/strategy.")
        return

    model_name = df["model_name"].iloc[0]
    version = df["version"].iloc[0]

    base_path = (
        os.path.join(
            "results",
            "ensembles",
            dataset_name,
            strategy_name,
            f"ensemble_ver{version}",
        )
        if is_ensemble
        else os.path.join(
            "results",
            dataset_name,
            strategy_name,
            shorten_model_name(model_name),
            f"ver{version}",
        )
    )

    metrics_path = os.path.join(base_path, "evaluation_results_metrics.json")
    summary_path = os.path.join(base_path, "evaluation_results_summary.json")
    if df["judge_filter_id"].notna().any() and df["judge_filter_id"].iloc[0]:
        judge_model_id = df["judge_filter_id"].iloc[0]
        metrics_path = os.path.join(
            base_path, f"evaluation_results_metrics_{judge_model_id}.json"
        )
        summary_path = os.path.join(
            base_path, f"evaluation_results_summary_{judge_model_id}.json"
        )
    metrics = load_json_safe(metrics_path)
    summary = load_json_safe(summary_path)

    if not metrics:
        st.warning(f"Metrics not found at {metrics_path}")
        return

    col1, col2 = st.columns(2)
    with col1:
        st.markdown(
            f"<h3>Total samples: {safe_display(metrics.get('total', 0))}</h3>",
            unsafe_allow_html=True,
        )

    with col2:
        if dataset_name.lower() == "bp" and strategy_name.lower() != "classification":
            points_map = {"right": 1.0, "somewhat right": 0.5}

            total_points = 0
            total_count = 0
            for label, count in metrics.get("bin_counts", {}).items():
                pts_per_item = points_map.get(label.lower(), 0)
                total_points += pts_per_item * count
                total_count += count

            st.markdown(
                f"<h3>Points: {safe_display(total_points):.2f}</h3>",
                unsafe_allow_html=True,
            )

        else:
            st.markdown(
                f"<h3>Accuracy: {safe_display(metrics.get('accuracy')):.2%}</h3>",
                unsafe_allow_html=True,
            )

    if not is_ensemble:
        headers = ["Score", "Count", "Average Confidence", "Median Confidence"]
        for c, h in zip(st.columns([3, 1, 2, 2]), headers):
            with c:
                render_cell(h, bold=True)

    for label, count in metrics.get("bin_counts", {}).items():
        avg = metrics.get("avg_confidence", {}).get(label, 0)
        med = metrics.get("median_confidence", {}).get(label, 0)

        cols = st.columns([3, 2, 2, 2])
        with cols[0]:
            render_cell(label)
        with cols[1]:
            render_cell(count)
        with cols[2]:
            render_cell(round(avg, 2))
        with cols[3]:
            render_cell(round(med, 2))

    if not summary:
        st.warning(f"Summary not found at {summary_path}")
        return

    render_cell("Data completeness", bold=True)

    for key, label in [
        ("answers_completeness", "Answers"),
        ("key_completeness", "Answer key"),
    ]:
        section = summary.get(key)
        if not section:
            continue

        expected = section.get("expected_num_samples", 0)
        missing = section.get("num_missing_problem_ids", 0)
        available = expected - missing
        coverage = available / expected if expected else 0

        cols = st.columns([3, 2, 2, 2])
        with cols[0]:
            render_cell(label)
        with cols[1]:
            render_cell(f"{available}/{expected}")
        with cols[2]:
            render_cell(f"{coverage:.1%}")
        with cols[3]:
            render_cell(f"{missing} missing")

        missing_ids = section.get("missing_problem_ids", [])
        if missing_ids:
            st.markdown(
                f"<p style='font-size:18px; color:#d62728;'>"
                f"<i>Missing Problem IDs: {', '.join(missing_ids)}</i></p>",
                unsafe_allow_html=True,
            )


# model configuration


def show_single_model_config(model_name, dataset_name, strategy_name, version):
    st.subheader("Configuration")

    metadata_path = os.path.join(
        "results",
        dataset_name,
        strategy_name,
        shorten_model_name(model_name),
        f"ver{version}",
        "metadata.json",
    )
    metadata = load_json_safe(metadata_path)
    if not metadata:
        st.warning(f"Metadata not found at {metadata_path}")
        return

    tech_cfg = (
        load_json_safe(
            os.path.join("src", "technical", "configs", "models_config.json")
        )
        or {}
    )
    param_sets = tech_cfg.get(model_name, {}).get("param_sets", {})

    ps = metadata.get("param_set_number")
    temperature = param_sets.get(str(ps), {}).get("temperature") if ps else None

    render_markdown("Model name", metadata.get("model"))
    render_markdown("Temperature", temperature)
    render_markdown("Dataset", metadata.get("dataset"))
    render_markdown("Dataset Category", metadata.get("config", {}).get("category"))
    render_markdown("Task Type", metadata.get("config", {}).get("task_type"))
    render_markdown("Strategy", metadata.get("strategy"))

    st.markdown("### Prompts")
    prompts = [
        ("problem_description_prompt", 150),
        ("question_prompt", 180),
        ("sample_answer_prompt", 100),
        ("example_prompt", 150),
        ("describe_prompt", 150),
        ("describe_example_prompt", 150),
        ("contrast_example_prompt", 150),
    ]

    for key, height in prompts:
        value = metadata.get(key)
        if value:
            st.text_area(key.replace("_", " ").title(), value, height=height)

    with st.expander("Full experiment metadata"):
        st.json(metadata)

    with st.expander("Full technical model config"):
        st.json(tech_cfg.get(model_name, {}))


def show_ensemble_config(dataset_name, type_name, ensemble_version):
    st.subheader("Ensemble Configuration")

    config_path = os.path.join(
        "results",
        "ensembles",
        dataset_name,
        type_name,
        f"ensemble_ver{ensemble_version}",
        "ensemble_config.json",
    )
    metadata = load_json_safe(config_path)
    if not metadata:
        st.warning(f"Metadata not found at {config_path}")
        return

    tech_cfg = (
        load_json_safe(
            os.path.join("src", "technical", "configs", "models_config.json")
        )
        or {}
    )

    for k in ["ensemble_model", "dataset_name", "dataset_category", "task_type"]:
        render_markdown(k.replace("_", " ").title(), metadata.get(k))

    st.markdown("### Main Prompt")
    st.text_area(
        "Main Prompt", safe_display(metadata.get("main_prompt", "")), height=300
    )

    for mk in sorted(k for k in metadata if k.startswith("member_")):
        member = metadata[mk]
        st.markdown(f"#### {mk.replace('_',' ').title()}")

        model = member.get("model")
        param_sets = tech_cfg.get(model, {}).get("param_sets", {})
        ps = member.get("param_set_number")
        temperature = param_sets.get(str(ps), {}).get("temperature") if ps else None

        render_markdown("Model Name", model)
        render_markdown("Temperature", temperature)
        render_markdown("Strategy", member.get("strategy"))

        with st.expander(f"Full config for {mk.replace('_',' ').title()}"):
            st.json(member)

    with st.expander("Full Ensemble metadata"):
        st.json(metadata)


# problem × strategy table


def show_problem_strategy_table(
    df,
    dataset_name,
    outcome_col="score",
    problem_col="problem_id",
    strategy_col="strategy_name",
):

    st.subheader("Problem × Strategy Outcome Overview")

    df = df[df["dataset_name"] == dataset_name].copy()
    if df.empty:
        st.info("No data for selected dataset.")
        return

    df[outcome_col] = df[outcome_col].fillna("No answer provided")
    color_map = get_present_colors(df, score_col=outcome_col)

    pivot = (
        df.pivot(index=problem_col, columns=strategy_col, values=outcome_col)
        .sort_index()
        .fillna("")
    )

    styled = pivot.style.applymap(
        lambda v: f"background-color:{color_map[v]}; color:black"
        if v in color_map
        else ""
    )

    st.dataframe(styled, height=450, use_container_width=True)



import streamlit as st
import pandas as pd
import argparse
import os

from iqbench.visualisation.data_handling import load_results, prepare_columns
from iqbench.visualisation.ui_helpers import multiselect_filter, map_strategy_column_name
from iqbench.visualisation.display import *


class StreamlitVisualiser:
    def __init__(self, csv_path: str):
        self.csv_path = csv_path
        self.df = load_results(csv_path)
        if not self.df.empty:
            self.df = prepare_columns(self.df)

    def run(self):
        setup_layout()

        if self.df.empty:
            st.info("No CSV loaded. Please provide a valid CSV file.")
            return

        df_model = self._select_model()

        if df_model is None:
            return
        df_model = self._select_judge_model(df_model) if df_model is not None else None

        if df_model is None:
            return

        df_single, is_ensemble, strategy_col = self._apply_filters(df_model)
        if df_single is None:
            return

        self._show_overview(df_single, strategy_col)
        self._show_dataset_section(df_model, is_ensemble, strategy_col)

    def _select_model(self) -> pd.DataFrame | None:
        options = sorted(self.df["filter_id"].unique())
        selected_id = st.selectbox("Select Model", options)
        df_model = self.df[self.df["filter_id"] == selected_id]
        if df_model.empty:
            st.info("No data for selected model.")
            return None
        return df_model

    def _select_judge_model(self, df) -> pd.DataFrame | None:
        if "judge_filter_id" not in df.columns:
            return df

        options = sorted(
            x for x in df["judge_filter_id"].unique() if pd.notna(x) and x != ""
        )

        if not options:
            return df

        selected_id = st.selectbox("Select Judge Model", options, index=0)

        df_model = df[
            (df["judge_filter_id"] == selected_id)
            | (df["judge_filter_id"].isna())
            | (df["judge_filter_id"] == "")
        ]

        return df_model

    def _apply_filters(
        self, df_model: pd.DataFrame
    ) -> tuple[pd.DataFrame | None, bool | None, str | None]:
        df = df_model.copy()
        df["dataset_name"] = df["dataset_name"].fillna("Unknown Dataset")
        df = multiselect_filter(df, "dataset_name", "Select Dataset(s)")
        if df.empty:
            return None, None, None

        is_ensemble = bool(df["ensemble"].iloc[0])
        strategy_col = map_strategy_column_name(is_ensemble)
        df = multiselect_filter(
            df,
            strategy_col,
            "Select Type(s)" if is_ensemble else "Select Strategy(ies)",
        )
        if df.empty:
            st.info("No data matches the selected filters.")
            return None, None, None

        return df, is_ensemble, strategy_col

    def _show_overview(self, df_single: pd.DataFrame, strategy_col: str) -> None:
        show_csv_preview(df_single)
        plot_judged_answers(df_single, strategy_col=strategy_col)
        plot_confidence(df_single, strategy_col=strategy_col)

    def _show_dataset_section(
        self, df_model: pd.DataFrame, is_ensemble: bool, strategy_col: str
    ) -> None:
        st.subheader("Details for Chosen Dataset")
        selected_dataset = st.selectbox(
            "Select Dataset", sorted(df_model["dataset_name"].unique())
        )
        df_dataset = df_model[df_model["dataset_name"] == selected_dataset]

        show_problem_strategy_table(
            df_dataset, dataset_name=selected_dataset, strategy_col=strategy_col
        )

        st.subheader(
            "Evaluation Summary for Chosen Dataset and "
            + ("Type" if is_ensemble else "Strategy")
        )
        selected_strategy = st.selectbox(
            "Select Strategy" if not is_ensemble else "Select Type",
            sorted(df_dataset[strategy_col].unique()),
        )
        display_evaluation_summary(
            df_dataset,
            dataset_name=selected_dataset,
            strategy_name=selected_strategy,
            strategy_col=strategy_col,
            is_ensemble=is_ensemble,
        )

        st.divider()
        st.subheader("Sample Problem Details")
        selected_problem_id = st.selectbox(
            "Select Problem ID",
            sorted(
                df_dataset[df_dataset[strategy_col] == selected_strategy][
                    "problem_id"
                ].unique()
            ),
        )
        show_chosen_problem(
            df_dataset,
            problem_id=selected_problem_id,
            dataset_name=selected_dataset,
            strategy_name=selected_strategy,
            strategy_col=strategy_col,
        )

        if not is_ensemble:
            show_single_model_config(
                dataset_name=selected_dataset,
                model_name=df_dataset["model_name"].iloc[0],
                strategy_name=selected_strategy,
                version=df_dataset["version"].iloc[0],
            )
        else:
            show_ensemble_config(
                dataset_name=selected_dataset,
                type_name=selected_strategy,
                ensemble_version=df_dataset["version"].iloc[0],
            )


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "csv_path", nargs="?", default=os.path.join("results", "all_results_concat.csv")
    )
    args = parser.parse_args()
    print("Using CSV path:", args.csv_path)

    visualiser = StreamlitVisualiser(csv_path=args.csv_path)
    visualiser.run()


if __name__ == "__main__":
    main()



from traitlets import List
import os
from typing import Optional, Dict, Any
from PIL import Image


from iqbench.technical.content import ImageContent, TextContent
from iqbench.strategies.strategy_base import StrategyBase
from iqbench.technical.response_schema import DescriptionResponseSchema, ResponseSchema
from iqbench.models.vllm import VLLM
from iqbench.technical.configs.dataset_config import DatasetConfig
from iqbench.technical.utils import get_field


class DescriptiveStrategy(StrategyBase):
    def __init__(
        self,
        dataset_name: str,
        model: VLLM,
        dataset_config: DatasetConfig,
        results_dir: str,
        strategy_name: str,
        prompt_number: int,
        param_set_number: Optional[int] = None,
    ):
        super().__init__(
            dataset_name,
            model,
            dataset_config,
            results_dir,
            strategy_name,
            prompt_number=prompt_number,
            param_set_number=param_set_number,
        )

        self.descriptions_prompt = self.get_prompt(f"describe", self.prompt_number)
        self.describe_example_prompt = self.get_prompt(
            f"describe_example", self.prompt_number
        )
        self.descriptions_path = os.path.join(self.results_dir, "descriptions.json")

    def _execute_problem(
        self, problem_id: str
    ) -> list[Dict[str, str], str, Optional[Dict[str, str]]]:
        """
        Executes the logic for a single descriptive problem.
        """

        descriptions = []
        problem_descriptions_dict = {}

        for i in range(self.config.num_choices):
            letter = chr(65 + i)

            if self.config.category == "BP":
                choice_image_input = self.get_choice_image(problem_id, image_index=i)
                index_key = i
            elif self.config.category in {"standard", "choice_only"}:
                choice_image_input = self.get_choice_image(
                    problem_id, image_index=letter
                )
                index_key = letter

            descriptions_prompt_with_example = (
                f"{self.descriptions_prompt}\n{self.describe_example_prompt}"
            )

            contents_to_send_descriptions = [
                TextContent(descriptions_prompt_with_example),
                ImageContent(choice_image_input),
            ]

            description_response = self.model.ask(
                contents_to_send_descriptions, schema=DescriptionResponseSchema
            )

            raw_description = get_field(description_response, "description", None)
            self.logger.debug(f"Description for choice {index_key}: {raw_description}")

            if raw_description:
                problem_descriptions_dict[index_key] = raw_description

            descriptions.append(
                f"{index_key}: {raw_description}" if raw_description else None
            )

        all_descriptions_text = "\n\n".join(
            [desc for desc in descriptions if desc is not None]
        )

        prompt = f"{self.main_prompt}\nDescriptions:\n{all_descriptions_text}\n{self.example_prompt}"

        if self.config.category in {"BP", "choice_only"}:
            contents_to_send = [TextContent(prompt)]

        elif self.config.category == "standard":
            image_input = self.get_question_image(problem_id)
            if image_input is None:
                self.logger.error(
                    f"Could not get question image for problem {problem_id}. Skipping image content."
                )
                contents_to_send = [TextContent(prompt)]
            else:
                contents_to_send = [TextContent(prompt), ImageContent(image_input)]

        response = self.model.ask(contents_to_send, schema=ResponseSchema)

        return response, problem_id, problem_descriptions_dict



import streamlit as st
import pandas as pd
import os
import json


def multiselect_filter(df: pd.DataFrame, column: str, label: str) -> pd.DataFrame:
    options = sorted(df[column].dropna().unique())
    selected = st.multiselect(label, options, default=options)
    return df[df[column].isin(selected)] if selected else df


def safe_display(value):
    if pd.isna(value) or value == "":
        return "-"
    return value


def map_strategy_column_name(is_ensemble):
    if is_ensemble:
        return "type_name"
    else:
        return "strategy_name"


def shorten_model_name(model_name: str) -> str:
    parts = model_name.split("/")
    if len(parts) >= 3:
        short_model_name = parts[1]
    elif len(parts) == 2:
        short_model_name = parts[1]
    else:
        short_model_name = model_name
    short_model_name = short_model_name.replace("/", "_")
    return short_model_name


def render_markdown(label: str, value, font_size=18):
    st.markdown(
        f"<p style='font-size:{font_size}px;'>"
        f"<b>{safe_display(label)}</b>: {safe_display(value)}"
        f"</p>",
        unsafe_allow_html=True,
    )



import os
import json
from pathlib import Path
import traceback
from typing import Dict, Tuple, Any
import PIL
from dotenv import load_dotenv
from huggingface_hub import snapshot_download, login
from iqbench.technical.configs.dataset_config import DatasetConfig
from iqbench.preprocessing.processor_factory import ProcessorFactory
from iqbench.technical.utils import get_config_path
import logging
import shutil


class DataModule:
    """Main data processing module."""

    def __init__(
        self,
        load_from_hf: bool = False,
    ):
        self.config_path = Path(get_config_path("dataset"))
        self.load_from_hf = load_from_hf
        self.logger = logging.getLogger("DataModule")
        self.configs: Dict[str, DatasetConfig] = {}
        self.raw_config_dicts: Dict[str, Dict[str, Any]] = {}
        self.configs, self.raw_config_dicts = self.load_configs()
        self.processors = {}
        load_dotenv()

    def load_configs(
        self,
    ) -> Tuple[Dict[str, DatasetConfig], Dict[str, Dict[str, Any]]]:
        """Load all dataset configurations."""
        try:
            with open(self.config_path, "r") as f:
                config_dict = json.load(f)

            configs = {}
            raw_cfgs = {}
            for name, cfg in config_dict.items():
                try:
                    configs[name] = DatasetConfig.from_dict(cfg)
                    raw_cfgs[name] = cfg
                except Exception as e:
                    self.logger.error(f"Error loading config for {name}: {e}")

            self.logger.info(f"Loaded {len(configs)} dataset configurations")
            return configs, raw_cfgs

        except Exception as e:
            self.logger.error(f"Error loading config file {self.config_path}: {e}")
            raise

    def setup(self) -> None:
        """Set up all processors."""
        from iqbench.preprocessing.standard_sheetmaker import StandardSheetMaker

        sheet_maker = StandardSheetMaker()

        for dataset_name, config in self.configs.items():
            try:
                self.processors[dataset_name] = ProcessorFactory.create_processor(
                    config,
                    sheet_maker if config.category != "BP" else None,
                )
                self.logger.debug(f"Created processor for {dataset_name}")
            except Exception as e:
                self.logger.error(f"Error creating processor for {dataset_name}: {e}")

        self.logger.info(f"Set up {len(self.processors)} processors")

    def download_datasets(self) -> None:
        """Download datasets from HuggingFace."""
        datasets_to_download = {
            name: config for name, config in self.configs.items() if config.hf_repo_id
        }

        if not datasets_to_download:
            self.logger.info("No datasets configured for HuggingFace download")
            return

        self.logger.info(
            f"Found {len(datasets_to_download)} datasets to download from HuggingFace"
        )

        hf_token = os.getenv("HF_API_TOKEN")
        if not hf_token:
            self.logger.error("HF_API_TOKEN not found in environment variables")
            raise ValueError("HF_API_TOKEN required for downloading from HuggingFace")

        try:
            login(token=hf_token)
            self.logger.info("Successfully logged in to HuggingFace")
        except Exception as e:
            self.logger.error(f"Failed to login to HuggingFace: {e}")
            raise

        downloaded = set()
        for name, config in datasets_to_download.items():
            if config.hf_repo_id not in downloaded:
                try:
                    self.download_from_hf(config.hf_repo_id, config.hf_repo_type)
                    downloaded.add(config.hf_repo_id)
                except Exception as e:
                    self.logger.error(f"Failed to download {name} {config.hf_repo_id}: {e}")

        self.logger.info(f"Downloaded {len(downloaded)} datasets from HuggingFace")

    def download_from_hf(self, repo_id: str, repo_type: str = "dataset") -> None:
        """Download or resume downloading a dataset from HuggingFace."""
        data_path = os.path.join("data_raw", repo_id.split("/")[-1])
        os.makedirs(data_path, exist_ok=True)

        self.logger.info(f"Checking existing data for {repo_id} in {data_path}...")

        try:
            self.logger.info(
                f"Starting (or resuming) download of {repo_id} to {data_path}"
            )
            snapshot_download(
                repo_id=repo_id,
                repo_type=repo_type,
                local_dir=str(data_path),
                max_workers=1,
            )
            self.logger.info(f"Download (or resume) complete for {repo_id}")

        except Exception as e:
            self.logger.error(f"Error downloading {repo_id}: {e}", exc_info=True)
            raise

    def check_dataset_counts(self) -> bool:
        """
        Check actual downloaded sample counts against 'expected_num_samples' from config.

        Returns:
            bool: True if all counts match, False otherwise.
        """
        self.logger.info("Checking dataset sample counts...")
        all_match = True

        mismatches = []

        for dataset_name, config in self.configs.items():
            raw_cfg = self.raw_config_dicts[dataset_name]
            expected = raw_cfg.get("expected_num_samples")

            if expected is None:
                self.logger.debug(
                    f"No 'expected_num_samples' for {dataset_name}, skipping check."
                )
                continue

            raw_data_path = Path(config.data_folder)
            actual = 0

            try:
                if not raw_data_path.exists():
                    self.logger.warning(
                        f"Data folder for {dataset_name} not found: {raw_data_path}"
                    )
                else:
                    actual = len(
                        [
                            p
                            for p in os.listdir(raw_data_path)
                            if os.path.isdir(os.path.join(raw_data_path, p))
                        ]
                    )
            except Exception as e:
                self.logger.error(
                    f"Error counting samples for {dataset_name} in {raw_data_path}: {e}"
                )

            if actual == expected:
                self.logger.info(
                    f"{dataset_name}: Found {actual} / {expected} samples (Match)"
                )
            else:
                msg = f"{dataset_name}: Found {actual} / {expected} samples (MISMATCH)"
                self.logger.error(msg)
                mismatches.append(msg)
                all_match = False

        if all_match:
            self.logger.info("All dataset counts match expected values.")
            return True
        else:
            self.logger.critical("DATASET VALIDATION FAILED")
            self.logger.critical("The following datasets have incomplete downloads:")
            for m in mismatches:
                self.logger.critical(f" - {m}")
            self.logger.critical(
                "Please run the download again to fetch missing files."
            )
            return False

    def process_all(self) -> None:
        """Process all datasets."""
        total_start_time = logging.time if hasattr(logging, "time") else None

        for dataset_name, processor in self.processors.items():
            self.logger.info(f"{'='*60}")
            self.logger.info(f"Processing dataset: {dataset_name}")
            self.logger.info(f"{'='*60}")

            try:
                processor.process()
            except Exception as e:
                self.logger.error(
                    f"Failed to process {dataset_name}: {e}", exc_info=True
                )

        for root, _, files in os.walk("data/"):
            for file in files:
                if file.lower().endswith((".jpg", ".jpeg", ".bmp", ".gif", ".tiff")):
                    file_path = os.path.join(root, file)
                    img = PIL.Image.open(file_path)
                    png_file_path = os.path.splitext(file_path)[0] + ".png"
                    img.save(png_file_path, "PNG")
                    os.remove(file_path)

        self.logger.info(f"{'='*60}")
        self.logger.info("All datasets processing complete")
        self.logger.info(f"{'='*60}")

    def verify_outputs(self) -> None:
        """
        Checks all processed datasets for missing solutions or annotations
        by scanning the output 'data/' directory.
        """
        self.logger.info("Starting output verification...")

        for dataset_name, config_data in self.raw_config_dicts.items():
            self.logger.info(f"--- Checking: {dataset_name} ---")

            problem_dir = Path(os.path.join("data", dataset_name, "problems"))
            if not problem_dir.exists():
                self.logger.warning(
                    f"  No 'problems' directory found for {dataset_name}."
                )
                continue

            try:
                processed_problems = set(
                    p.name for p in problem_dir.iterdir() if p.is_dir()
                )
                if not processed_problems:
                    self.logger.info(
                        f"  No processed problems found for {dataset_name}."
                    )
                    continue
                self.logger.info(
                    f"  Found {len(processed_problems)} processed problems."
                )
            except Exception as e:
                self.logger.error(f"  Failed to list processed problems: {e}")
                continue

            json_dir = os.path.join("data", dataset_name, "jsons")
            solution_keys = set()
            annotation_keys = set()

            solutions_path = os.path.join(json_dir, f"{dataset_name}_solutions.json")

            if os.path.exists(solutions_path):
                try:
                    with open(solutions_path, "r", encoding="utf-8") as f:
                        solutions_data = json.load(f)
                    solution_keys = set(solutions_data.keys())
                except Exception as e:
                    self.logger.error(
                        f"  Failed to load solutions file {solutions_path}: {e}"
                    )
            else:
                self.logger.warning(f"  No solutions file found at {solutions_path}")

            has_annotations_config = bool(config_data.get("annotations_folder"))

            if has_annotations_config:
                annotations_path = os.path.join(
                    json_dir, f"{dataset_name}_annotations.json"
                )
                if os.path.exists(annotations_path):
                    try:
                        with open(annotations_path, "r", encoding="utf-8") as f:
                            annotations_data = json.load(f)
                        annotation_keys = set(annotations_data.keys())
                    except Exception as e:
                        self.logger.error(
                            f"  Failed to load annotations file {annotations_path}: {e}"
                        )
                else:
                    self.logger.warning(
                        f"  No annotations file found at {annotations_path}"
                    )

            missing_solutions = processed_problems - solution_keys
            if not os.path.exists(solutions_path):
                pass
            elif missing_solutions:
                self.logger.warning(
                    f"  Found {len(missing_solutions)} problems missing solutions."
                )
                if len(missing_solutions) < 10:
                    self.logger.warning(
                        f"    Missing: {sorted(list(missing_solutions))}"
                    )
            else:
                self.logger.info(
                    f"  Solutions check passed (all {len(processed_problems)} problems have solutions)."
                )

            if not has_annotations_config:
                self.logger.info("  (Annotations not configured for this dataset)")
            else:
                missing_annotations = processed_problems - annotation_keys
                if not annotations_path.exists():
                    pass
                elif missing_annotations:
                    self.logger.warning(
                        f"  Found {len(missing_annotations)} problems missing annotations."
                    )
                    if len(missing_annotations) < 10:
                        self.logger.warning(
                            f"    Missing: {sorted(list(missing_annotations))}"
                        )
                else:
                    self.logger.info(
                        f"  Annotations check passed (all {len(processed_problems)} problems have annotations)."
                    )

        self.logger.info("--- Output check complete ---")

    def run(self) -> None:
        """Run the complete data processing pipeline."""
        self.logger.info("Starting data processing pipeline")

        if self.load_from_hf:
            self.logger.info("Download from HuggingFace enabled")
            try:
                self.download_datasets()
            except Exception as e:
                self.logger.error(f"Dataset download failed: {e}")
                return
        else:
            self.logger.info(
                "Using existing local data (HuggingFace download disabled)"
            )

        self.logger.info("Checking downloaded dataset counts...")
        try:
            if not self.check_dataset_counts():
                self.logger.error(
                    "Pipeline stopped due to dataset count mismatches. "
                    "Please re-run with download enabled or fix the data manually."
                )
                return
        except Exception as e:
            self.logger.error(f"Failed to check dataset counts: {e}", exc_info=True)
            return

        self.logger.info("Setting up processors...")
        try:
            self.setup()
        except Exception as e:
            self.logger.error(f"Processor setup failed: {e}")
            return

        self.logger.info("Processing all datasets...")
        self.process_all()

        self.logger.info("Verifying processed outputs...")
        self.verify_outputs()

        self.logger.info("Pipeline execution complete!")



import os
from typing import Optional, Dict


from iqbench.strategies.strategy_base import StrategyBase
from iqbench.technical.content import ImageContent, TextContent
from iqbench.technical.response_schema import ResponseSchema


class ClassificationStrategy(StrategyBase):
    def _execute_problem(
        self, problem_id: str
    ) -> list[Dict[str, str], str, Optional[Dict[str, str]]]:

        image_path = self.get_classification_panel(problem_id)
        prompt = f"{self.main_prompt}\n{self.example_prompt}"

        contents_to_send = [TextContent(prompt), ImageContent(image_path)]

        response = self.model.ask(contents=contents_to_send, schema=ResponseSchema)

        return response, problem_id, None  # None for descriptions



from abc import ABC, abstractmethod
import json
import string
from typing import Any, List, Optional, Type
import logging
from pydantic import BaseModel
import pandas as pd
import os
from iqbench.technical.utils import (
    get_results_directory,
    get_dataset_config,
    get_ensemble_directory,
    shorten_model_name,
)
from pathlib import Path

logger = logging.getLogger(__name__)


class EvaluationBase(ABC):
    def __init__(self, judge_model_object=None, judge_model_name=None):
        self.judge_model_object = judge_model_object
        self.judge_model_name = judge_model_name

    @abstractmethod
    def evaluate_single_answer(self, *args, **kwargs) -> float:
        pass

    @abstractmethod
    def evaluate(self, *args, **kwargs):
        pass

    def run_evaluation(
        self,
        dataset_name: str,
        version: str,
        strategy_name: Optional[str] = None,
        model_name: Optional[str] = None,
        ensemble: bool = False,
        type_name: Optional[str] = None,
        evaluation_output_path: str = "evaluation_results",
        concat: bool = True,
        output_all_results_concat_path: str = "all_results_concat",
    ):
        results_dir, model_name = self._resolve_results_dir(
            dataset_name, version, strategy_name, model_name, ensemble, type_name
        )

        default_dir = results_dir.split("results")[0] + "results"
        output_all_results_concat_path = os.path.join(
            default_dir, f"{output_all_results_concat_path}.csv"
        )

        answers_path, key_path = self._get_evaluation_paths(
            dataset_name=dataset_name, results_dir=results_dir, ensemble=ensemble
        )

        if not results_dir or not answers_path or not key_path:
            raise ValueError("Evaluation cannot proceed due to missing paths.")

        answers_df = self._load_answers(answers_path)
        descriptions = self._load_descriptions(results_dir, ensemble)

        expected_num_samples = get_dataset_config(dataset_name).expected_num_samples
        dataset_category = get_dataset_config(dataset_name).category
        summary_answers = self.check_completeness(
            answers_df, expected_num_samples, descriptions
        )
        logger.info(f"Answers DataFrame Completeness Summary: {summary_answers}")

        output_df = answers_df.copy()
        output_df["score"] = ""
        output_df["key"] = ""

        key_dict, summary_key = self._load_key_and_prepare_summary(
            key_path, expected_num_samples
        )
        logger.info(f"Key DataFrame Completeness Summary: {summary_key}")

        self.evaluate(
            output_df=output_df, key_dict=key_dict, dataset_category=dataset_category
        )

        self._write_summary_and_metrics(
            results_dir=results_dir,
            evaluation_output_path=evaluation_output_path,
            summary_answers=summary_answers,
            summary_key=summary_key,
            output_df=output_df,
        )

        if concat:
            self.append_to_all_results_concat(
                results_df=output_df,
                all_results_concat_path=output_all_results_concat_path,
                dataset_name=dataset_name,
                model_name=model_name,
                strategy_name=strategy_name,
                version=version,
                type_name=type_name,
                ensemble=ensemble,
            )

    @abstractmethod
    def calculate_metrics(self, *args, **kwargs) -> dict:
        pass

    def check_completeness(self, df, expected_num_samples, descriptions=None) -> dict:
        summary = {}

        summary["row_ids_with_any_missing"] = df.loc[
            df.isna().any(axis=1), "problem_id"
        ].tolist()
        summary["row_ids_fully_missing"] = df.loc[
            df.isna().all(axis=1), "problem_id"
        ].tolist()

        summary["missing_count_per_column"] = df.isna().sum().to_dict()
        summary["missing_ratio_per_column"] = df.isna().mean().to_dict()

        summary["expected_num_samples"] = expected_num_samples

        num_digits = len(str(expected_num_samples))
        expected_ids = {str(i).zfill(num_digits) for i in range(expected_num_samples)}
        actual_ids = set(df["problem_id"].tolist())

        missing_ids = sorted(expected_ids - actual_ids)
        summary["missing_problem_ids"] = missing_ids
        summary["num_missing_problem_ids"] = len(missing_ids)

        if descriptions:
            json_summary = {}

            incomplete_ids = {}
            for outer_id, inner_dict in descriptions.items():
                missing_inner = sorted(
                    [k for k, v in inner_dict.items() if not v or str(v).strip() == ""]
                )
            if missing_inner:
                incomplete_ids[outer_id] = missing_inner

            json_summary["problem_ids_with_missing_descriptions"] = incomplete_ids
            json_summary["num_problem_ids_with_missing_descriptions"] = len(
                incomplete_ids
            )

            summary["descriptions_completeness"] = json_summary

        return summary

    def append_to_all_results_concat(
        self,
        results_df: pd.DataFrame,
        all_results_concat_path: str,
        dataset_name: str,
        model_name: str = None,
        strategy_name: Optional[str] = None,
        version: Optional[str] = None,
        type_name: Optional[str] = None,
        ensemble: bool = False,
    ):

        results_df["dataset_name"] = dataset_name
        results_df["model_name"] = model_name
        results_df["strategy_name"] = strategy_name
        results_df["version"] = version
        results_df["type_name"] = type_name

        if ensemble:
            results_df["ensemble"] = True
            results_df["seed"] = int(version) % 10
            results_df["ens_members_config_number"] = int(version) // 10 % 10
        else:
            results_df["ensemble"] = False

        if os.path.exists(all_results_concat_path):
            existing_df = pd.read_csv(
                all_results_concat_path, dtype={"problem_id": str}, encoding="utf-8"
            )
            combined_df = pd.concat([existing_df, results_df], ignore_index=True)
        else:
            combined_df = results_df

        meta_cols = [
            "judge_rationale",
            "judge_model_name",
            "judge_model_param_set",
            "dataset_name",
            "model_name",
            "strategy_name",
            "version",
        ]

        for col in meta_cols:
            if col not in combined_df.columns:
                combined_df[col] = ""

        other_cols = [c for c in combined_df.columns if c not in meta_cols]
        final_order = other_cols + meta_cols

        key_cols = [
            "problem_id",
            "dataset_name",
            "judge_model_name",
            "judge_model_param_set",
            "version",
            "ensemble",
            "model_name",
            "strategy_name",
            "type_name",
        ]

        for col in key_cols:
            if col in combined_df.columns:
                combined_df[col] = combined_df[col].fillna("").astype(str).str.strip()

        combined_df = combined_df.drop_duplicates(subset=key_cols, keep="last")

        combined_df = combined_df[final_order]
        combined_df.to_csv(all_results_concat_path, index=False)

    def _get_evaluation_paths(
        self, dataset_name: str, results_dir: str, ensemble: bool = False
    ):

        if ensemble:
            answers_path = os.path.join(results_dir, "ensemble_results.csv")
        else:
            answers_path = os.path.join(results_dir, "results.csv")

        dataset_config = get_dataset_config(dataset_name)
        if "classification" in results_dir and dataset_config.category == "BP":
            key_path = os.path.join(
                "data", dataset_name, "jsons", "classification_solutions.json"
            )
        else:
            key_path = os.path.join(
                "data", dataset_name, "jsons", f"{dataset_name}_solutions.json"
            )

        if not results_dir or not os.path.exists(results_dir):
            logger.error(
                f"Results directory is not provided or does not exist {results_dir}."
            )
            results_dir = None

        if not answers_path or not os.path.exists(answers_path):
            logger.error(
                f"Answers path is not provided or does not exist {answers_path}."
            )
            answers_path = None

        if not key_path or not os.path.exists(key_path):
            logger.error(f"Key path is not provided or does not exist {key_path}.")
            key_path = None

        return answers_path, key_path

    def _resolve_results_dir(
        self,
        dataset_name: str,
        version: str,
        strategy_name: Optional[str],
        model_name: Optional[str],
        ensemble: bool,
        type_name: Optional[str],
    ):
        if ensemble:
            if type_name is None:
                raise ValueError("type_name is required when ensemble=True")
            results_dir = get_ensemble_directory(
                dataset_name=dataset_name,
                type_name=type_name,
                version=version,
                create_dir=False,
            )
            ensemble_config_path = os.path.join(results_dir, "ensemble_config.json")
            if os.path.exists(ensemble_config_path):
                with open(ensemble_config_path, "r") as f:
                    metadata = json.load(f)
                    model_name = metadata.get("ensemble_model", None)
            else:
                logger.warning(
                    "Ensemble config file not found, model_name will be set to None."
                )
                model_name = None
        else:
            if strategy_name is None:
                raise ValueError("strategy_name is required when ensemble=False")
            if model_name is None:
                raise ValueError("model_name is required when ensemble=False")
            results_dir = get_results_directory(
                dataset_name=dataset_name,
                strategy_name=strategy_name,
                model_name=model_name,
                version=version,
                create_dir=False,
            )
        return results_dir, model_name

    def _load_answers(self, answers_path):
        df = pd.read_csv(answers_path, dtype={"problem_id": str}, encoding="utf-8")
        return df

    def _load_descriptions(self, results_dir, ensemble):
        descriptions = None
        if not ensemble:
            metadata_path = os.path.join(results_dir, "metadata.json")
            with open(metadata_path, "r") as f:
                metadata = json.load(f)
            if metadata["strategy"] in ["descriptive", "contrastive"]:
                descriptions_path = os.path.join(results_dir, "descriptions.json")
                with open(descriptions_path, "r") as f:
                    descriptions = json.load(f)
        return descriptions

    def _load_key_and_prepare_summary(self, key_path, expected_num_samples):
        with open(key_path, "r") as f:
            key_dict = json.load(f)
        key_df = pd.DataFrame(
            {"problem_id": list(key_dict.keys()), "answer": list(key_dict.values())}
        )
        summary_key = self.check_completeness(key_df, expected_num_samples)
        return key_dict, summary_key

    def _write_summary_and_metrics(
        self,
        results_dir,
        evaluation_output_path,
        summary_answers,
        summary_key,
        output_df,
    ):
        if hasattr(self, "judge_model_object"):
            model_suffix = f"_{shorten_model_name(self.judge_model_name)}"
            param_set = getattr(self, "judge_param_set_number", None)
            if param_set is not None:
                model_suffix += f"_{param_set}"
            else:
                model_suffix += "_1"
        else:
            model_suffix = ""

        output_summaries_path = os.path.join(
            results_dir, f"{evaluation_output_path}_summary{model_suffix}.json"
        )
        with open(output_summaries_path, "w") as summary_file:
            json.dump(
                {
                    "answers_completeness": summary_answers,
                    "key_completeness": summary_key,
                },
                summary_file,
                indent=4,
            )
        logger.info(f"Summaries saved to {output_summaries_path}")

        metrics = self.calculate_metrics(output_df)
        metrics_path = os.path.join(
            results_dir, f"{evaluation_output_path}_metrics{model_suffix}.json"
        )
        with open(metrics_path, "w") as metrics_file:
            json.dump(metrics, metrics_file, indent=4)
        logger.info(f"Metrics saved to {metrics_path}")

        output_path = os.path.join(
            results_dir, f"{evaluation_output_path}{model_suffix}.csv"
        )
        output_df.to_csv(output_path, index=False)
        logger.info(f"Results saved to {output_path}")



from iqbench.preprocessing.base_processor import BaseProcessor
from iqbench.technical.configs.dataset_config import DatasetConfig
from iqbench.preprocessing.standard_processor import StandardProcessor
from iqbench.preprocessing.bongard_processor import BongardProcessor


class ProcessorFactory:
    """Factory for creating appropriate processor instances."""

    @staticmethod
    def create_processor(config: DatasetConfig, sheet_maker=None) -> BaseProcessor:
        """Create the appropriate processor based on dataset category."""
        if config.category == "BP":
            return BongardProcessor(config)
        elif config.category in ["standard", "choice_only"]:
            if sheet_maker is None:
                raise ValueError("sheet_maker is required for standard processors")
            return StandardProcessor(config, sheet_maker)
        else:
            raise ValueError(f"Unknown category: {config.category}")



from pydantic import BaseModel, Field
from typing import List
from enum import Enum


class ResponseSchema(BaseModel):
    answer: str = Field(..., description="The model's answer to the question.")
    confidence: float = Field(
        ...,
        description="The model's confidence in its answer, ranging from 0.0 to 1.0.",
    )
    rationale: str = Field(..., description="The model's rationale for its answer.")

class DescriptionResponseSchema(BaseModel):
    description: str = Field(
        ..., description="A detailed description of the provided image content."
    )

class EvaluationLabel(str, Enum):
    RIGHT = "Right"
    SOMEWHAT_RIGHT = "Somewhat right"
    UNCLEAR = "Unclear"
    SOMEWHAT_WRONG = "Somewhat wrong"
    WRONG = "Wrong"


class BongardEvaluationSchema(BaseModel):
    reasoning: str = Field(
        ...,
        description="A one-sentence reasoning explaining the decision based on the rubric.",
    )

    similarity_label: EvaluationLabel = Field(
        ..., description="The specific categorical label representing the similarity."
    )


class GeneralEnsembleSchema(BaseModel):
    final_answer: str = Field(
        ..., description="The final answer chosen by the ensemble method."
    )
    rationale: str = Field(
        ..., description="The rationale behind the ensemble's final answer."
    )



from dataclasses import dataclass
from typing import Optional, Dict, Any


@dataclass
class DatasetConfig:
    """Configuration for a dataset processor."""

    image_format: str
    task_type: str
    category: str
    data_folder: str
    num_choices: int
    choice_images_folder: str
    regex_choice_number: str
    shuffle: bool
    hf_repo_id: Optional[str] = None
    hf_repo_type: Optional[str] = "dataset"
    question_images_folder: Optional[str] = None
    answer_images_folder: Optional[str] = None
    annotations_folder: Optional[str] = None
    regex_answer_number: Optional[str] = None
    solutions_folder: Optional[str] = None
    true_idx: Optional[int] = None
    expected_num_samples: Optional[int] = None

    @classmethod
    def from_dict(cls, config_dict: Dict[str, Any]) -> "DatasetConfig":
        """Create config from dictionary."""
        return cls(**config_dict)



import os
import json
from typing import List, Optional, Dict, Any
from PIL import Image, ImageDraw
from iqbench.preprocessing.base_processor import BaseProcessor
from iqbench.preprocessing.standard_sheetmaker import StandardSheetMaker
from iqbench.technical.configs.dataset_config import DatasetConfig


class BongardProcessor(BaseProcessor):
    """Specialized processor for Bongard Problems."""

    def process(self) -> None:
        """Process all Bongard problems."""
        processed_count = 0
        skipped_count = 0
        error_count = 0

        problems = [
            p
            for p in os.listdir(self.raw_data_path)
            if os.path.isdir(os.path.join(self.raw_data_path, p))
        ]

        self.logger.info(f"Found {len(problems)} problems to process")
        classification_solutions = {}
        for problem_id in problems:
            problem_path = os.path.join(self.raw_data_path, problem_id)

            if not os.path.isdir(problem_path):
                continue

            problem_id_standardized = self.standardize_problem_id(problem_id)

            if self.is_already_processed(problem_id_standardized):
                self.logger.debug(
                    f"Problem {problem_id_standardized} already processed, skipping"
                )
                skipped_count += 1
                continue

            try:
                images = self.load_choice_images(problem_id)

                if None in images:
                    self.logger.warning(
                        f"Skipping problem {problem_id} due to missing images"
                    )
                    error_count += 1
                    continue

                self.save_refactored_images(
                    problem_id_standardized, images, letters=False
                )

                sheets = self.generate_bongard_sheet(images)

                self.save_sheet(problem_id_standardized, sheets["normal"])
                self.save_sheet(
                    problem_id_standardized, sheets["switched"], switched=True
                )

                sheetmaker = StandardSheetMaker()

                (
                    classification_panel,
                    answer_label,
                    _,
                ) = sheetmaker.generate_question_sheet_from_images(
                    images=[sheets["normal"], sheets["switched"]],
                    shuffle_answers=True, 
                    true_answer_index=0,
                )
                self.save_sheet(
                    problem_id_standardized,
                    classification_panel,
                    classification_panel=True,
                )

                classification_solutions[problem_id_standardized] = answer_label

                processed_count += 1
                self.logger.debug(
                    f"Successfully processed problem {problem_id_standardized}"
                )

            except Exception as e:
                self.logger.error(
                    f"Error processing problem {problem_id}: {e}", exc_info=True
                )
                error_count += 1
        self.save_json(classification_solutions, "classification_solutions.json")

        self.logger.info(
            f"Processing complete: {processed_count} processed, "
            f"{skipped_count} skipped (already processed), "
            f"{error_count} errors"
        )

        if (
            processed_count > 0
            or not self.get_output_dir("jsons").joinpath("bp_solutions.json").exists()
        ):
            self.process_solutions()
        else:
            self.logger.info(
                "No new problems processed and solutions already exist, skipping solutions processing"
            )

    def load_choice_images(self, problem_id: str) -> List[Optional[Image.Image]]:
        """Load all 12 images for a Bongard problem."""
        images = []
        problem_path = os.path.join(self.raw_data_path, problem_id)

        for i in range(self.config.num_choices):
            image_path = os.path.join(problem_path, f"{i}.png")
            try:
                images.append(Image.open(image_path).convert("RGB"))
            except Exception as e:
                self.logger.error(f"Error loading image {image_path}: {e}")
                images.append(None)

        return images

    def generate_bongard_sheet(
        self,
        images: List[Image.Image],
        spacing: int = 10,
        margin: int = 20,
        border_thickness: int = 2,
        space_between_panels: int = 40,
    ) -> Dict[str, Image.Image]:
        """Generate two Bongard problem sheets: normal and with 5↔11 switched."""
        if len(images) != 12:
            raise ValueError("Exactly 12 images are required for Bongard problems.")

        def _make_sheet(imgs: List[Image.Image]) -> Image.Image:
            """Internal helper to build one sheet."""
            max_width = max(img.width for img in imgs)
            resized_images = []
            for img in imgs:
                if img.width != max_width:
                    ratio = max_width / img.width
                    new_height = int(img.height * ratio)
                    img = img.resize((max_width, new_height))
                resized_images.append(img)

            col_width = max_width

            max_row_heights = [
                max(
                    resized_images[0].height,
                    resized_images[3].height,
                    resized_images[6].height,
                    resized_images[9].height,
                ),
                max(
                    resized_images[1].height,
                    resized_images[4].height,
                    resized_images[7].height,
                    resized_images[10].height,
                ), 
                max(
                    resized_images[2].height,
                    resized_images[5].height,
                    resized_images[8].height,
                    resized_images[11].height,
                ),
            ]

            total_width = (
                col_width * 4 + spacing * 3 + space_between_panels + 2 * margin
            )

            total_height = sum(max_row_heights) + spacing * 2 + 2 * margin

            sheet = Image.new("RGB", (total_width, total_height), color=(255, 255, 255))
            draw = ImageDraw.Draw(sheet)

            x_coords = [
                margin,
                margin + col_width + spacing,
                margin + col_width * 2 + spacing * 2 + space_between_panels,
                margin + col_width * 3 + spacing * 3 + space_between_panels,
            ]

            y_coords = [
                margin,
                margin + max_row_heights[0] + spacing,
                margin + max_row_heights[0] + max_row_heights[1] + spacing * 2,
            ]

            image_grid_indices = [
                [0, 3, 6, 9], 
                [1, 4, 7, 10], 
                [2, 5, 8, 11],
            ]

            for r_idx in range(3):
                for c_idx in range(4):
                    img_idx = image_grid_indices[r_idx][c_idx]
                    img = resized_images[img_idx]
                    x_pos = x_coords[c_idx]
                    y_pos = y_coords[r_idx]

                    sheet.paste(img, (x_pos, y_pos))

                    draw.rectangle(
                        [x_pos, y_pos, x_pos + img.width - 1, y_pos + img.height - 1],
                        outline="black",
                        width=border_thickness,
                    )

            return sheet

        normal_sheet = _make_sheet(images)

        switched_images = images.copy()
        switched_images[5], switched_images[11] = (
            switched_images[11],
            switched_images[5],
        )
        switched_sheet = _make_sheet(switched_images)

        return {"normal": normal_sheet, "switched": switched_sheet}

    def process_solutions(self) -> None:
        """Process and renumber solutions from raw data."""
        raw_solutions_path = os.path.join(self.raw_data_path, "bp_solutions.json")

        if not raw_solutions_path.exists():
            self.logger.warning(f"Solutions file not found: {raw_solutions_path}")
            return

        try:
            with open(raw_solutions_path, "r", encoding="utf-8") as f:
                answers_data = json.load(f)

            renumbered_data = {
                f"{int(k):03d}": answers_data[k]
                for k in sorted(answers_data.keys(), key=lambda x: int(x))
            }

            self.save_json(renumbered_data, "bp_solutions.json")

        except Exception as e:
            self.logger.error(f"Error processing solutions: {e}", exc_info=True)



import pandas as pd
import random

from typing import Optional, Any, List

from iqbench.ensemble.ensemble_base import EnsembleBase
from iqbench.technical.response_schema import GeneralEnsembleSchema
from iqbench.technical.content import TextContent
from iqbench.technical.utils import get_field, get_dataset_config
from iqbench.models.llm_judge import LLMJudge
from string import Template


class ConfidenceEnsemble(EnsembleBase):
    def __init__(
        self,
        dataset_name,
        members_configuration,
        skip_missing=True,
        type_name="confidence",
        judge_model: Optional[LLMJudge] = None,
        prompt_number: Optional[int] = 1,
        version: Optional[int] = None,
        seed: Optional[int] = 42,
    ):
        super().__init__(
            dataset_name,
            members_configuration,
            skip_missing,
            type_name,
            prompt_number,
            version=version,
            seed=seed,
        )
        if get_dataset_config(dataset_name).category == "BP":
            self.llm = judge_model if judge_model is not None else LLMJudge()
            self.config["ensemble_model"] = self.llm.get_model_name()
        else:
            self.config[
                "ensemble_model"
            ] = "No judge model needed for this type of dataset."

    def evaluate_single_problem(self, problem_id):
        rationale = None
        single_problem_df = self.answers[
            self.answers["problem_id"] == problem_id
        ].copy()

        if single_problem_df.empty:
            self.logger.warning(f"No answers for problem {problem_id}")
            return None

        if self.dataset_config.category == "BP":
            answer_list = single_problem_df["answer"].tolist()
            confidence_list = single_problem_df["confidence"].tolist()

            final_answer, rationale, raw_response = self.evaluate_confidence_using_llm(
                answer_list, confidence_list
            )
            return final_answer, rationale, raw_response

        else:
            if "answer" not in single_problem_df.columns:
                self.logger.error(f"'answer' column missing for problem {problem_id}")
                return None
            
            avg_confidence = single_problem_df.groupby("answer")["confidence"].mean()
            max_avg_confidence = avg_confidence.max()
            tied_answers = avg_confidence[
                avg_confidence == max_avg_confidence
            ].index.tolist()
            most_popular_answer = random.choice(tied_answers)
            return most_popular_answer, rationale, None

    def evaluate_confidence_using_llm(self, answer_list, confidence_list):

        all_answers_str = "\n".join(
            f"- {ans} (confidence: {conf})"
            for ans, conf in zip(answer_list, confidence_list)
        )
        prompt_filled = self._get_filled_prompt(all_answers=all_answers_str)

        schema = GeneralEnsembleSchema
        response = self.llm.ask(
            [TextContent(prompt_filled)],
            schema=schema,
        )

        final_answer = get_field(response, "final_answer")
        rationale = get_field(response, "rationale")
        return final_answer, rationale, response



from pathlib import Path
from typing import Any
import pandas as pd
import json
import logging
import os
import importlib.resources as pkg_resources

from iqbench.evaluation.evaluation_base import EvaluationBase
from iqbench.models.llm_judge import LLMJudge
from iqbench.technical.response_schema import BongardEvaluationSchema
from iqbench.technical.utils import shorten_model_name


class EvaluationWithJudge(EvaluationBase):
    def __init__(
        self,
        judge_model_name: str = "mistralai/Mistral-7B-Instruct-v0.3",
        judge_model_object: Any = None,
        judge_param_set_number: int = None,
        prompt: str = None,
        prompt_number: int = 1,
        prompt_path: str = None,
    ):
        self.logger = logging.getLogger(__name__)
        self.prompt_number = prompt_number
        self.judge_model_name = judge_model_name
        self.judge_param_set_number = judge_param_set_number

        if prompt is not None:
            self.prompt = prompt

        elif prompt_path is not None:
            if not os.path.exists(prompt_path):
                error_msg = f"Prompt file not found: {prompt_path}. Check if prompt path is correct."
                raise ValueError(error_msg)
            try:
                with open(prompt_path, "r", encoding="utf-8") as file:
                    self.prompt = file.read()
            except (OSError, IOError) as e:
                error_msg = f"Error reading prompt file at {prompt_path}: {e}."
                self.logger.exception(error_msg)
                raise ValueError(error_msg) from e
        else:
            rel_path = f"prompts/evaluation/evaluation_bongard_{self.prompt_number}.txt"
            fallback_rel_path = "prompts/evaluation/evaluation_bongard_1.txt"
            
            try:
                package_root = pkg_resources.files("iqbench")
                resource = package_root.joinpath(rel_path)

                if not resource.exists():
                    self.logger.warning(
                        f"Prompt {self.prompt_number} not found. Attempting to default to prompt 1."
                    )
                    resource = package_root.joinpath(fallback_rel_path)

                if not resource.exists():
                    error_msg = f"Internal prompt resource not found: {rel_path}. Default version 1 also missing."
                    raise ValueError(error_msg)

                self.prompt = resource.read_text(encoding="utf-8")

            except (OSError, IOError, Exception) as e:
                error_msg = f"Error accessing internal prompt resource: {e}."
                self.logger.exception(error_msg)
                raise ValueError(error_msg) from e

        if judge_model_object is not None:
            self.judge_model_object = judge_model_object
            self.judge_model_name = judge_model_object.get_model_name()

        else:
            self.logger.info(f"Initializing judge model: {self.judge_model_name}")
            self.judge_model_object = LLMJudge(
                model_name=self.judge_model_name,
                param_set_number=self.judge_param_set_number,
            )

    def evaluate_single_answer(
        self,
        answer: str,
        key: str,
        response_schema: BongardEvaluationSchema,
    ):
        return self.judge_model_object.evaluate_similarity(
            prompt=self.prompt, answer=answer, key=key, response_schema=response_schema
        )

    def evaluate(
        self,
        output_df: pd.DataFrame,
        key_dict: dict,
        dataset_category: str,
        stop_after_evaluation: bool = False,
    ):

        for index, row in output_df.iterrows():
            answer = row.get("answer")
            id_ = str(row["problem_id"])

            if id_ not in key_dict:
                self.logger.info(f"ID {id_} not found in key file.")
                output_df.at[index, "score"] = "Problem id not found in key"
                output_df.at[index, "key"] = "Key missing"
                continue

            if dataset_category.lower() == "bp":
                left_rule, right_rule = key_dict[id_]
                key = f"{left_rule} vs. {right_rule}"
            else:
                key = key_dict[id_]

            if answer is None or pd.isna(answer) or answer.strip() == "":
                output_df.at[index, "score"] = "No answer provided"
                output_df.at[index, "key"] = key
                continue

            score, judge_rationale = self.evaluate_single_answer(
                answer=answer,
                key=key,
                response_schema=BongardEvaluationSchema,
            )

            output_df.at[index, "key"] = key

            if score is None:
                output_df.at[index, "score"] = "LLM evaluation failed"
                continue

            if judge_rationale is None:
                output_df.at[index, "judge_rationale"] = "LLM reasoning missing"
            output_df.at[index, "score"] = score
            output_df.at[index, "judge_rationale"] = judge_rationale
        output_df["judge_model_name"] = shorten_model_name(self.judge_model_name)
        output_df["judge_model_param_set"] = (
            self.judge_param_set_number
            if self.judge_param_set_number is not None
            else 1
        )

        if stop_after_evaluation:
            self.judge_model_object.stop()

    def calculate_metrics(self, evaluated_df):
        total = len(evaluated_df)

        if "score" in evaluated_df.columns:
            correct = (evaluated_df["score"] == "Right").sum()
            accuracy = correct / total if total > 0 else 0.0
        else:
            accuracy = 0.0

        if "score" in evaluated_df.columns:
            grouped = evaluated_df.groupby("score")
            bin_counts = grouped.size().to_dict()
        else:
            bin_counts = {}

        if "confidence" in evaluated_df.columns and "score" in evaluated_df.columns:
            bin_counts = grouped["confidence"].size().to_dict()
            avg_confidence = grouped["confidence"].mean().to_dict()
            median_confidence = grouped["confidence"].median().to_dict()
        else:
            avg_confidence = {}
            median_confidence = {}

        return {
            "total": total,
            "bin_counts": bin_counts,
            "accuracy": accuracy,
            "avg_confidence": avg_confidence,
            "median_confidence": median_confidence,
        }



from dataclasses import dataclass
from typing import Optional


@dataclass
class EvaluationConfig:
    """Configuration for evaluation process."""

    dataset_name: str
    version: str

    strategy_name: Optional[str] = None
    model_name: Optional[str] = None
    ensemble: bool = False
    type_name: Optional[str] = None

    evaluation_output_path: str = "evaluation_results"
    concat: bool = True
    output_all_results_concat_path: str = "all_results_concat"

    judge_model_name: Optional[str] = "mistralai/Mistral-7B-Instruct-v0.3"
    judge_param_set_number: Optional[int] = None
    judge_model_object: Optional[object] = None
    prompt_number: int = 1



import subprocess
import time
from typing import List, Optional, Dict, Type, Any
import portpicker
import requests
import logging
import sys
import os
from pydantic import BaseModel

from iqbench.technical.content import Content, ImageContent, TextContent
from iqbench.technical.prompt_formatter import PromptFormatter
from iqbench.models.vllm import VLLM
from iqbench.technical.utils import get_field

logger = logging.getLogger(__name__)


class LLMJudge(VLLM):
    def __init__(
        self,
        model_name: str = "mistralai/Mistral-7B-Instruct-v0.3",
        param_set_number: Optional[int] = None,
    ):
        super().__init__(
            model_name=model_name,
            param_set_number=param_set_number,
        )

        self.judge_mode = "text_only"
        logger.info(
            f"Initialized LLMJudge for text-only evaluation with model {model_name}"
        )

    def evaluate_similarity(
        self,
        prompt: str,
        answer: str,
        key: str,
        response_schema: Optional[Type[BaseModel]],
    ):
        try:
            prompt = f"{prompt}\n" f"Answer: {answer}\n" f"Key Answer: {key}\n"

            if response_schema:
                response = self.ask([TextContent(prompt)], response_schema)

            elif self.cpu_local_testing:
                response = self.ask([TextContent(prompt)])

            else:
                response = self.ask([TextContent(prompt)])

            similarity_label = get_field(
                response, "similarity_label", "No similarity label provided."
            )
            reasoning = get_field(response, "reasoning", "No reasoning provided.")

            return similarity_label, reasoning

        except Exception as e:
            logger.error(f"Similarity evaluation failed: {e}")
            return None, None
